use fantoccini::{elements::Element, Client, ClientBuilder, Locator};
use lazy_static::lazy_static;
use regex;

/// Holds data about stackoverflow answers
#[derive(Debug, Clone)]
struct Answer {
    upvotes: u32,
    author: String,
    content: String,
}

/// Get all answers from a stackoverflow domain. No error handling is done so get ready to either
/// check your input or "note: run with `RUST_BACKTRACE=1` environment variable to display a
/// backtrace"
async fn get_answers(c: &Client, url: &str) -> Vec<Answer> {
    // first, go to the Wikipedia page for Foobar
    c.goto(url).await.unwrap();

    let answer_loc = c.find_all(Locator::Css(".answer")).await.unwrap();
    let mut out_answers = vec![];
    for answer in answer_loc {
        let text = answer.text().await.unwrap();

        let score = text
            .clone()
            .split('\n')
            .collect::<Vec<&str>>()
            .get(0)
            .unwrap()
            .parse::<u32>()
            .unwrap();
        let content = text;

        out_answers.push(Answer {
            upvotes: score,
            content,
            author: "unimplemented".to_string(),
        });
    }

    out_answers
}

async fn get_top_links(c: &Client, pages: u16) -> anyhow::Result<Vec<String>> {
    let mut answers = vec![];
    for page in 1..=pages {
        c.goto(
            format!(
                "https://stackoverflow.com/questions?tab=votes&page={}",
                page
            )
            .as_str(),
        )
        .await
        .unwrap();

        let finds = c.find_all(Locator::Css(".s-link")).await.unwrap();
        for find in finds {
            if find
                .attr("href")
                .await
                .unwrap()
                .unwrap()
                .contains("/questions/")
            {
                answers.push(find.attr("href").await.unwrap().unwrap());
            }
        }
    }

    Ok(answers)
}

#[tokio::main]
async fn main() {
    let c: Client = ClientBuilder::native()
        .connect("http://localhost:4444")
        .await
        .expect("failed to connect to WebDriver");
    let links = get_top_links(&c, 5).await.unwrap();
    let mut answers = vec![];
    for link in links {
        answers.append(
            &mut get_answers(&c, format!("https://stackoverflow.com{}", link).as_str()).await,
        );
    }
    c.close().await.unwrap();
}
